// TODO : créer un message pour les debugs de la carte d'alimentation (4 float)

#include <LiquidCrystal_I2C.h>

#include <Servo.h>

#include <ros.h>
#include <std_msgs/Bool.h>
#include <std_msgs/Int32.h>
#include <ctrlpan_msgs/alim_stat.h>

// Servo object declaration
Servo flagServo; 
#define FLAG_UNFOLDED_ANGLE 170
#define FLAG_FOLDED_ANGLE 0

Servo armServo; //pour manche à air
#define ARM_UNFOLDED_ANGLE 90
#define ARM_FOLDED_ANGLE 0

// I2C : (if not working : try inverse sda and scl pin)
// todo : remove nucleo jumpers to use these pins.
#define I2C_SDA A5
#define I2C_SCL A4

// Pin declaration
const int flagServoPin = D6;
const int armServoPin = D4;

const int pullCordPin = A6;
const int switchPin = A7;
const int buttonPin = D10;
const int buttonLedPin = D12;

const int voltage_twelvePin = A3;
const int voltage_fivePin = A0;
const int current_twelvePin = A2;
const int current_fivePin = A1;
#define VOLT12_RATIO 1
#define VOLT5_RATIO 1
#define CURR5_RATIO 1
#define CURR12_RATIO 1

const int bauStatusPin = D8;
const int bauSoftPin = D9;


// Flag state
bool flagStatus;

// BAU state
bool bauSoftStatus;
bool bauStatus;

// Arm state
bool armStatus;



// Current team (either TEAM_YELLOW or TEAM_BLUE)
bool team;
bool buttonStatus;

// Last received estimated score
int estimatedScore = 0;

/**
 * Tirette (avec gestion du rebond)
 */

int pullCordState;            // the current reading from the input pin
int lastPullCordStatus = LOW; // the previous reading from the input pin

unsigned long lastDebounceTime = 0; // the last time the output pin was toggled
unsigned long debounceDelay = 50;   // the debounce time; increase if the output flickers

unsigned long lastMessageTime = 0;
unsigned int const MSG_SENDING_INTERVAL = 200;

ros::NodeHandle nh;

std_msgs::Bool teamMsg;
std_msgs::Bool buttonStatusMsg;
std_msgs::Bool pullCordStatusMsg;
std_msgs::Bool bauStatusMsg;
ctrlpan_msgs::alim_stat alimStatMsg;

ros::Publisher teamStatusPub("team", &teamMsg);
ros::Publisher buttonStatusPub("button_status", &buttonStatusMsg);
ros::Publisher bauStatusPub("BAU_status", &bauStatusMsg);
ros::Publisher pullCordStatusPub("pull_cord_status", &pullCordStatusMsg);
ros::Publisher alimStatPub("alim_stat", &alimStatMsg);

void estimatedScoreCallback(const std_msgs::Int32 &estimatedScore);
ros::Subscriber<std_msgs::Int32> estimatedScoreSub("estimated_score", &estimatedScoreCallback);

void flagStatusChange(const std_msgs::Bool &flagStatusMsg);
ros::Subscriber<std_msgs::Bool> flagStatusSub("flag_state", &flagStatusChange);

void armStatusChange(const std_msgs::Bool &armStatusMsg);
ros::Subscriber<std_msgs::Bool> armStateSub("arm_state", &armStatusChange);

void bauSoftSet(const std_msgs::Bool &bauSoftSetMsg);
ros::Subscriber<std_msgs::Bool> bauSoftSub("bau_state", &bauSoftSet);

// LCD screen
LiquidCrystal_I2C lcd(0x27, 16, 2);

void refreshScreen()
{
  lcd.clear();

  lcd.setCursor(0, 0);
  lcd.print("Score=");
  lcd.setCursor(6, 0);
  lcd.print(estimatedScore);

  lcd.setCursor(0, 1);
  lcd.print(team ? "Team=JAUNE" : "Team=BLEU");
  lcd.setCursor(12, 1);
  lcd.print(pullCordState ? "GO" : "STOP");
  lcd.setCursor(13, 0);
  lcd.print(buttonStatus ? "ON" : "OFF");
}



void setup()
{

  // INIT Wire on : A4 => SDA  ||  A5 => SCL
	Wire.begin(I2C_SDA, I2C_SCL);


  flagServo.attach(flagServoPin);
  flagServo.write(0);

  armServo.attach(armServoPin);
  armServo.write(0);

  pinMode(pullCordPin, INPUT_PULLUP);
  pinMode(switchPin, INPUT_PULLUP);
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(buttonLedPin, OUTPUT);

  pinMode(bauSoftPin, OUTPUT);
  pinMode(bauStatusPin, INPUT);

  digitalWrite(buttonLedPin, HIGH);

  lcd.init();
  lcd.backlight();

  team = digitalRead(switchPin);
  buttonStatus = digitalRead(buttonPin);

  nh.initNode();
  nh.advertise(teamStatusPub);
  nh.advertise(buttonStatusPub);
  nh.advertise(pullCordStatusPub);
  nh.advertise(bauStatusPub);
  nh.advertise(alimStatPub);

  nh.subscribe(estimatedScoreSub);
  nh.subscribe(flagStatusSub);
  nh.subscribe(armStateSub);
  nh.subscribe(bauSoftSub);

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Waiting ROS...");

  while (!nh.connected())
  {
    nh.spinOnce();
    delay(MSG_SENDING_INTERVAL);
  }

  refreshScreen();
}

// Appelé quand le score est mis à jour (message de score reçu sur l'Arduino depuis la raspi)
void estimatedScoreCallback(const std_msgs::Int32 &estimatedScoreMsg)
{
  estimatedScore = estimatedScoreMsg.data;
  refreshScreen();
}

// Appelé quand l'état du drapeau est mis à jour
void flagStatusChange(const std_msgs::Bool &flagStatusMsg)
{
  flagStatus = flagStatusMsg.data;
}

// Appelé quand l'état du BAU est mis à jour
void bauSoftSet(const std_msgs::Bool &bauSoftSetMsg)
{
    bauSoftStatus = bauSoftSetMsg.data;
}

// Appelé quand l'état du bras (manche à air) est mis à jour
void armStatusChange(const std_msgs::Bool &armStatusMsg)
{
    armStatus = armStatusMsg.data;
}



void manage_supply_card_data(){

  int volt12_valueADC = analogRead(voltage_twelvePin);
  int volt5_valueADC = analogRead(voltage_fivePin);
  int curr5_valueADC = analogRead(current_fivePin);
  int curr12_valueADC = analogRead(current_twelvePin);

  float volt12_value = (float) volt12_valueADC * VOLT12_RATIO;
  float volt5_value = (float) volt5_valueADC * VOLT5_RATIO;
  float curr5_value = (float) curr5_valueADC * CURR5_RATIO;
  float curr12_value = (float) curr12_valueADC * CURR12_RATIO;

  alimStatMsg.voltage_12 = volt12_value;
  alimStatMsg.voltage_5 = volt5_value;
  alimStatMsg.current_12 = curr12_value;
  alimStatMsg.current_5 = curr5_value;

}


void loop()
{
  // TODOOOOOOO : CHANGE

  if (!nh.connected())
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Waiting ROS...");

    while (!nh.connected())
    {
      nh.spinOnce();
      delay(MSG_SENDING_INTERVAL);
    }

    refreshScreen();
  }

  // DEBUT PARTIE TIRETTE

  int pullCordReading = digitalRead(pullCordPin);

  // If the switch changed, due to noise or pressing:
  if (pullCordReading != lastPullCordStatus)
  {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }

  if ((millis() - lastDebounceTime) > debounceDelay)
  {
    // whatever the reading is at, it's been there for longer than the debounce
    // delay, so take it as the actual current state:

    // if the button state has changed:
    if (pullCordReading != pullCordState)
    {
      pullCordState = pullCordReading;
      refreshScreen();
    }
  }

  lastPullCordStatus = pullCordReading;

  // FIN PARTIE TIRETTE

  if (!digitalRead(buttonPin) != buttonStatus)
  {
    // Appui bouton
    buttonStatus = !digitalRead(buttonPin);
    refreshScreen();
  }

  if (digitalRead(switchPin) != team)
  {
    //Changement d'équipe
    team = digitalRead(switchPin);
    refreshScreen();
  }

  bauStatus = digitalRead(bauStatusPin);

  if (millis() - lastMessageTime >= MSG_SENDING_INTERVAL)
  {
    teamMsg.data = team;
    teamStatusPub.publish(&teamMsg);

    pullCordStatusMsg.data = pullCordState;
    pullCordStatusPub.publish(&pullCordStatusMsg);

    buttonStatusMsg.data = buttonStatus;
    buttonStatusPub.publish(&buttonStatusMsg);

    bauStatusMsg.data = bauStatus;
    bauStatusPub.publish(&bauStatusMsg);

    manage_supply_card_data();
    alimStatPub.publish(&alimStatMsg);

    lastMessageTime = millis();
    nh.spinOnce();
  }

  if(flagStatus)
  {
    flagServo.write(FLAG_UNFOLDED_ANGLE);
  } else {
    flagServo.write(FLAG_FOLDED_ANGLE);
  }

  if(armStatus)
  {
    armServo.write(ARM_UNFOLDED_ANGLE);
  } else {
    armServo.write(ARM_FOLDED_ANGLE);
  }

  digitalWrite(bauSoftPin, bauSoftStatus);
}
